﻿using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;

public class playerController : MonoBehaviour
{
	const float randomChange = 0.5f;
	const float lerpValue = .6f;
	const float meetPercent = .1f;

	private Animator animator;
	private Transform t;
	private Camera cam;
	private float targetTime;
	private bool showGui = false;
	public bool isLobbyist = false;

	List<GUIStyle> styles;

	private Propaganda prop;
	private float curDelta = 0.0f;

	void initStyles() {
		styles = new List<GUIStyle> ();
		foreach (Color c in new Color[]{Color.white, Color.red, Color.blue, Color.green, Color.cyan, Color.black}) {
			Texture2D healthColor = new Texture2D (1,1);
			healthColor.SetPixel (0, 0, c);
			healthColor.wrapMode = TextureWrapMode.Repeat;
			healthColor.Apply ();
			GUIStyle style = new GUIStyle ();
			style.normal.background = healthColor;
			styles.Add (style);
		}
	}

	// Use this for initialization
	public void ParametrizedStart(Shader shader, Propaganda setProp)
	{
		animator = this.GetComponent<Animator>();
		t = this.GetComponent<Transform> ();
		cam = Camera.main;
		isLobbyist = true;

		prop = setProp;
		Renderer rend = GetComponent<Renderer>();
		rend.material.shader = shader;
		//rend.material.SetFloat ("_HueShift", float.Parse (row [6]));
		rend.material.SetTexture("_Mask", Resources.Load<Texture2D>("walk_cycle_mask"));

		initStyles ();
		animator.SetInteger("Direction", Mathf.FloorToInt (Random.value * 4.0f));
		targetTime = Random.value * 2.0f + 1.0f;
	}

	public void ParametrizedStart(string type)
	{
		animator = this.GetComponent<Animator>();
		t = this.GetComponent<Transform> ();
		cam = Camera.main;

		CSVReader reader = new CSVReader ();
		List<List<string>> csv = reader.parseCSV("Config/Stereotypes");
		foreach (List<string> row in csv) {
			if (row [0].Equals (type)) {
				prop = new Propaganda ();
				prop.empathy = float.Parse (row [1]);
				prop.reason = float.Parse(row[2]);
				prop.greed = float.Parse(row[3]);
				prop.fun = float.Parse(row[4]);
				prop.fear = float.Parse(row[5]);
				Shader sh = Shader.Find ("Custom/HSVShader");
				Renderer rend = GetComponent<Renderer>();
				rend.material.shader = sh;
				rend.material.SetFloat ("_HueShift", float.Parse (row [6]));
				rend.material.SetTexture("_Mask", Resources.Load<Texture2D>("walk_cycle_mask"));
			}
		}

		initStyles ();
		animator.SetInteger("Direction", Mathf.FloorToInt (Random.value * 4.0f));
		targetTime = Random.value * 2.0f + 1.0f;
		Debug.Log("" + t.position.z);
	}

	// Update is called once per frame
	void Update()
	{
		Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
		BoxCollider2D coll = GetComponent<BoxCollider2D>();
		if (coll.OverlapPoint (ray.origin)) {
			showGui = true;
		} else {
			showGui = false;
		}

		targetTime -= Time.deltaTime;
		if (targetTime <= 0.0f) {
			if (Random.value <= randomChange) {
				animator.SetInteger("Direction", Mathf.FloorToInt (Random.value * 4.0f));
			}
			targetTime = Random.value * 2.0f + 1.0f;
		}

		Vector3 pos = t.position;
		switch (animator.GetInteger("Direction")) {
		case 0:
			pos.y += lerpValue * Time.deltaTime;
			if (cam.WorldToScreenPoint(pos).y > Screen.height) {
				pos.y = cam.ScreenToWorldPoint(new Vector2(0, 0)).y;
			}
			break;
		case 1:
			pos.x += lerpValue * Time.deltaTime;
			if (cam.WorldToScreenPoint(pos).x > Screen.width) {
				pos.x = cam.ScreenToWorldPoint(new Vector2(0, 0)).x;
			}
			break;
		case 2:
			pos.y -= lerpValue * Time.deltaTime;
			if (cam.WorldToScreenPoint(pos).y < 0) {
				pos.y = cam.ScreenToWorldPoint(new Vector2(0, Screen.height)).y;
			}
			break;
		default:
			pos.x -= lerpValue * Time.deltaTime;
			if (cam.WorldToScreenPoint(pos).x < 0) {
				pos.x = cam.ScreenToWorldPoint(new Vector2(Screen.width, 0)).x;
			}
			break;
		}
		t.position = pos;
	}

	private float calcCollision(playerController other)
	{
		int maxIdx = -1;
		float maxDiff = -1.0f;

		for (int i = 0; i < 5; i++) {
			if (Mathf.Abs (propaganda.getByIndex(i) - other.propaganda.getByIndex(i)) > maxDiff) {
				maxDiff = Mathf.Abs (propaganda.getByIndex(i) - other.propaganda.getByIndex(i));
				maxIdx = i;
			}
		}
		if (maxIdx < 0) {
			return 0.0f;
		}

		if (isLobbyist || other.isLobbyist) {
			if (isLobbyist) {
				other.encounterPropaganda(this.propaganda);
			} else {
				encounterPropaganda(other.propaganda);
			}
			return 0.0f;
		}

		// "This" reference
		float multi = 0;
		multi = meetPercent * (delta < 0 ? 2.0f : 1.0f);
		if (propaganda.getByIndex (maxIdx) < other.propaganda.getByIndex (maxIdx)) {
			multi *= -1; // Lose diff
		}
		delta = delta + (multi * maxDiff);

		// "Other" reference
		multi = meetPercent * (other.delta < 0 ? 2.0f : 1.0f);
		if (isLobbyist)
			multi *= 5.0f;
		if (other.propaganda.getByIndex (maxIdx) < propaganda.getByIndex (maxIdx)) {
			multi *= -1; // Lose diff
		}
		other.delta = other.delta + (multi * maxDiff);

		return multi;
	}

	public float encounterPropaganda(Propaganda evt)
	{
		float resultVal = (evt * propaganda).getSum();
		if (resultVal != 0) {
			DeltaGuiFb go = gameObject.AddComponent<DeltaGuiFb> ();
			go.ParametrizedStart(resultVal);
			curDelta += resultVal;
		}
		return resultVal;
	}

	void OnCollisionEnter2D(Collision2D coll) {
		animator.SetInteger ("Direction", (animator.GetInteger ("Direction") + 2) % 4);
		if (coll.gameObject.tag.Equals ("Entity")) {
			float deltaChange = calcCollision (coll.gameObject.GetComponent<playerController> ());
			if (deltaChange != 0 && (t.position.sqrMagnitude < coll.gameObject.GetComponent<Transform> ().position.sqrMagnitude)) {
				DeltaGuiFb go = gameObject.AddComponent<DeltaGuiFb> ();
				go.ParametrizedStart(deltaChange);
			}
		}
	}

	public void OnGUI() {
		if (showGui) {
			Vector3 screenPos = cam.WorldToScreenPoint (new Vector3 (t.position.x, t.position.y, t.position.z));
			GUI.Box (new Rect (screenPos.x - 22, Screen.height - screenPos.y - 34, 40, 20), "");
			GUI.Label (new Rect (screenPos.x - 18, Screen.height - screenPos.y - 30, 32, 12), "", styles[0]);
			GUI.Label (new Rect (screenPos.x - 17, Screen.height - screenPos.y - 29, 30 * prop.empathy, 2), "", styles[1]);
			GUI.Label (new Rect (screenPos.x - 17, Screen.height - screenPos.y - 27, 30 * prop.reason, 2), "", styles[2]);
			GUI.Label (new Rect (screenPos.x - 17, Screen.height - screenPos.y - 25, 30 * prop.greed, 2), "", styles[3]);
			GUI.Label (new Rect (screenPos.x - 17, Screen.height - screenPos.y - 23, 30 * prop.fun, 2), "", styles[4]);
			GUI.Label (new Rect (screenPos.x - 17, Screen.height - screenPos.y - 21, 30 * prop.fear, 2), "", styles[5]);
		}
	}


	public Propaganda propaganda
	{
		get {
			return prop;
		}
		set {
			prop = value;
		}
	}

	public float delta 
	{
		get {
			return curDelta;
		}
		set {
			curDelta = value;
		}
	}

}
