﻿// Upgrade NOTE: replaced 'mul(UNITY_MATRIX_MVP,*)' with 'UnityObjectToClipPos(*)'

Shader "Custom/WhiteJacketShader" {
    Properties {
        _MainTex ("Texture", 2D) = "white" {}
        _Sat("Saturation", Float) = 1
        _Mask("Mask", 2D) = "white" {}
    }
    SubShader {
 
        Tags { "Queue"="Transparent" "IgnoreProjector"="True" "RenderType" = "Transparent" }
        ZWrite Off
        Blend SrcAlpha OneMinusSrcAlpha
        Cull Off
 
        Pass
        {
            CGPROGRAM
            #pragma vertex vert
            #pragma fragment frag
            #pragma target 2.0
 
            #include "UnityCG.cginc"

            float colToIntensity(float3 RGB)
            {
            	return .299*RGB.x + .587*RGB.y + .114*RGB.z;
            }

            struct v2f {
                float4  pos : SV_POSITION;
                float2  uv : TEXCOORD0;
            };
 
            float4 _MainTex_ST;
 
            v2f vert (appdata_base v)
            {
                v2f o;
                o.pos = UnityObjectToClipPos (v.vertex);
                o.uv = TRANSFORM_TEX(v.texcoord, _MainTex);
                return o;
            }
 
            sampler2D _MainTex;
            sampler2D _Mask;
            float _Sat;
 
            half4 frag(v2f i) : COLOR
            {
                half4 col = tex2D(_MainTex, i.uv);
                half4 maskCol = tex2D(_Mask, i.uv);
                if (maskCol.a <= 0.0f) return col;

                float intensity = colToIntensity(col);
                if (intensity < .15f) intensity = 1.0f-intensity;
                else intensity *= 2.0f;
            	return half4(intensity, intensity, intensity, col.a);
            }
            ENDCG
        }
    }
    Fallback "Particles/Alpha Blended"
}