﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.SceneManagement;


namespace Populism
{
    
    public class GameManager : MonoBehaviour
    {
		private string[] stereotypes = {"eco-activist", "pastafari", "hillbilly", "bankster", "rightwing extremist"};

        public GameObject gameManagerObject;
        public GameObject eventSystem;
		public GameObject PlayerHUD;

		private List<List<string>> lvlCfg;
		List<List<string>> csvValues;
		private List<AudioClip> sounds;
		AudioSource audio;
		List<GUIStyle> styles;
		bool showBalken = false;

        bool gameRunning = false;

        #region Entities

        // Stereotype reference list
		List<GameObject> stereoTypeEntities = new List<GameObject>();

        

		public void RegisterStereoType(GameObject stereoTypeEntity)
        {
			stereoTypeEntities.Add(stereoTypeEntity);
        }

        #endregion

        #region Time

        // Handle time

        public float defaultLevelTime = 100;

        public float timeLeft;

        public void UpdateGameTimer()
        {
            if (timeLeft > 0)
            {
                timeLeft -= Time.deltaTime;
            }

            if (timeLeft < 0)
            {
                timeLeft = 0;
                EndLevel();
            }
        }

        #endregion

        #region SceneManagement 

        public int currentLevel = 0;

        // Handle Menu

        public GameObject menu_AfterLevel; // Assign in inspector
        private bool isShowing;


        // Handle gamestart

		public void StartCurrentLevel(int num)
        {
			// Prepare Level
			PlayerHUD.SetActive (true);
			showBalken = true;

			PlayerHUD.GetComponent<PlayerHUD>().ResetPropagandaTexts(getPropagandaStrings ());

			ApplyGoalText ();

			List<string> cfg = null;
			foreach (List<string> lvl in lvlCfg) {
				if (int.Parse(lvl [0]) == num) {
					cfg = lvl;
					break;
				}
			}
			if (cfg != null) {
				GameObject objPrefab = Resources.Load<GameObject> ("walking_man");
				for (int col = 2; col <= 6; col++) {
					for (int i = 0; i < int.Parse (cfg [col]); i++) {
						Vector3 newPosGUI = new Vector3 (Random.value * Screen.width, Random.value * Screen.height, 11);
						GameObject go = Instantiate (objPrefab, Camera.main.ScreenToWorldPoint (newPosGUI), Quaternion.identity);
						go.GetComponent<playerController>().ParametrizedStart(stereotypes[col - 2]);
						RegisterStereoType (go);
					}
				}
			}

			// Start Level
            gameRunning = true;

            timeLeft = defaultLevelTime;

            isShowing = false;
            menu_AfterLevel.SetActive(isShowing);

            menu_AfterLevel.transform.Find("Info Text").GetComponent<Text>().text = "";
            menu_AfterLevel.transform.Find("Button_StartGame").gameObject.SetActive(false);
            menu_AfterLevel.transform.Find("Button_Retry").gameObject.SetActive(true);
            menu_AfterLevel.transform.Find("Button_NextLevel").gameObject.SetActive(false);


        }


        #endregion


        // Handle GameEnd

        public void EndLevel()
        {
            // count motivated entities

			PlayerHUD.SetActive (false);
			showBalken = false;

            gameRunning = false;

            int count = CountMotivatedEntities();

            menu_AfterLevel.SetActive(true);




            // see if threshhold is reached
            if ((float)count / stereoTypeEntities.Count > 0.8f)
            {
                // show next level menu
                menu_AfterLevel.transform.Find("Info Text").GetComponent<Text>().text = "Level Complete.";
                menu_AfterLevel.transform.Find("Button_Retry").gameObject.SetActive(false);
                menu_AfterLevel.transform.Find("Button_NextLevel").gameObject.SetActive(true);

				if (currentLevel == 3) 
				{
						GameCompleted ();
				}
            }
            else
            {
                // show retry menu
                menu_AfterLevel.transform.Find("Info Text").GetComponent<Text>().text = "Level Failed. Your propaganda was not strong enough...";
                menu_AfterLevel.transform.Find("Button_Retry").gameObject.SetActive(true);
                menu_AfterLevel.transform.Find("Button_NextLevel").gameObject.SetActive(false);
            }

			stereoTypeEntities.Clear ();
        }

        private void OnLevelWasLoaded(int level)
        {
            if (level > 0)
            {
				
				//gameObject.GetComponent<MouseListener> ().changeLevel ();
                StartCurrentLevel(level);
            }
        }

        public void GameCompleted()
        {
            menu_AfterLevel.transform.Find("Info Text").GetComponent<Text>().text = "Game Completed.";
			menu_AfterLevel.transform.Find ("Info Text").gameObject.SetActive (true);
            menu_AfterLevel.transform.Find("Button_StartGame").gameObject.SetActive(true);
            menu_AfterLevel.transform.Find("Button_Retry").gameObject.SetActive(false);
            menu_AfterLevel.transform.Find("Button_NextLevel").gameObject.SetActive(false);

            currentLevel = 0;
            SceneManager.LoadScene(currentLevel);
        }

        public int CountMotivatedEntities()
        {
            int count = 0;

            if (stereoTypeEntities.Count > 0)
            {
				foreach (GameObject attribute in stereoTypeEntities) {
					if (attribute.GetComponent<playerController>().delta >= 1.0f)
                    {
                        count++;
                    }
                }
            }

            return count;
        }



        // Use this for initialization
        void Start()
        {
            DontDestroyOnLoad(gameManagerObject);
			//PlayerHUD = GameObject.Find("PlayerHUD");
			CSVReader csv = new CSVReader();
			lvlCfg = csv.parseCSV(Resources.Load<TextAsset>("Config/Levels"));
			csvValues = csv.parseCSV ("Config/Propaganda");
			audio = gameObject.AddComponent<AudioSource >();
			sounds = new List<AudioClip> ();
			sounds.Add (Resources.Load <AudioClip>("papier"));
			sounds.Add (Resources.Load <AudioClip>("twitter"));
			sounds.Add (Resources.Load <AudioClip>("trump"));

			styles = new List<GUIStyle> ();
			foreach (Color c in new Color[]{Color.green, Color.red, Color.yellow}) {
				Texture2D healthColor = new Texture2D (1,1);
				healthColor.SetPixel (0, 0, c);
				healthColor.wrapMode = TextureWrapMode.Repeat;
				healthColor.Apply ();
				GUIStyle style = new GUIStyle ();
				style.normal.background = healthColor;
				styles.Add (style);
			}
        }

		public void playSound (int index) 
		{
			audio.PlayOneShot(sounds[index]);
		}

        private void MoveManagementToNextScene()
        {
            
        }

		void ApplyGoalText()
		{
			if (currentLevel == 1) {
				PlayerHUD.GetComponent<PlayerHUD> ().SetGoalText("bring swedish beavers near extinction.");

				PlayerHUD.transform.Find("Panel_Top/Text_CitizenLabel1").GetComponent<Text> ().text = "Red = Eco-activist.";
				PlayerHUD.transform.Find("Panel_Top/Text_CitizenLabel2").GetComponent<Text> ().text = "Purple = Hillbilly.";

			}
			if (currentLevel == 2) {
				PlayerHUD.GetComponent<PlayerHUD> ().SetGoalText("establish coal as fuel of the future.");

				PlayerHUD.transform.Find("Panel_Top/Text_CitizenLabel1").GetComponent<Text> ().text = "Green = Bankster.";
				PlayerHUD.transform.Find("Panel_Top/Text_CitizenLabel2").GetComponent<Text> ().text = "Purple = Hillbilly.";
			}
			if (currentLevel == 3) {
				PlayerHUD.GetComponent<PlayerHUD> ().SetGoalText("invest in a kickstarter to replace travel by plane with travel by being shot from a catapult.");

				PlayerHUD.transform.Find("Panel_Top/Text_CitizenLabel1").GetComponent<Text> ().text = "Red = Eco-activist.";
				PlayerHUD.transform.Find("Panel_Top/Text_CitizenLabel2").GetComponent<Text> ().text = "Yellow = Pastafari.";
				PlayerHUD.transform.Find("Panel_Top/Text_CitizenLabel3").GetComponent<Text> ().text = "Blue = Rightwing extremist.";
			}
		}


        #region ButtonEvents

        void SetupButtonEvents()
        {

        }

        public void buttonStartGamePressed()
        {
            Debug.Log("Start Pressed");
            menu_AfterLevel.transform.Find("Button_StartGame").gameObject.SetActive(false);
            menu_AfterLevel.transform.Find("Button_Retry").gameObject.SetActive(true);
            currentLevel++;
            SceneManager.LoadScene(currentLevel);
            
        }

        public void buttonRetryPressed()
        {
            SceneManager.LoadScene(currentLevel);
            
        }

        public void buttonNextLevelPressed()
        {
            currentLevel++;
            SceneManager.LoadScene(currentLevel);
            
        }

        public void buttonQuitPressed()
        {
            Application.Quit();
        }



        #endregion

		public void ShowCredits()
		{
			menu_AfterLevel.transform.Find("Text_Credits").gameObject.SetActive(true);
		}

        // Update is called once per frame
        void Update()
        {
            if(gameRunning)
            {
                UpdateGameTimer();

                if (Input.GetKeyDown("escape") && timeLeft > 0)
                {
                    isShowing = !isShowing;
                    menu_AfterLevel.SetActive(isShowing);
                }
            }
        }

		public List<string> getPropagandaValues(int idx)
		{
			int rowId = 6 * (currentLevel - 1) + idx;
			foreach (List<string> row in csvValues) {
				if (row [0].Equals ("" + rowId)) {
					return row.GetRange(2, row.Count - 2);
				}
			}
			return null;
		}

		public List<string> getPropagandaStrings()
		{
			List<string> result = new List<string> ();
			int rowId = 6 * (currentLevel - 1) + 1;
			int cnt = -1;
			foreach (List<string> row in csvValues) {
				if (cnt >= 6) break;
				if (row [0].Equals ("" + rowId)) cnt = 0;
				if (cnt >= 0) {
					result.Add (row [1]);
					cnt++;
				}

			}
			return result;
		}


		const int margin = 10;
		const int hgt = 10;
		void OnGUI() 
		{
			if (showBalken) {
				int count = 0;
				int countPos = 0;
				int countNeg = 0;


				if (stereoTypeEntities.Count > 0) {
					foreach (GameObject attribute in stereoTypeEntities) {
						count++;
						if (attribute.GetComponent<playerController> ().delta >= 1.0f)
							countPos++;
						else if (attribute.GetComponent<playerController> ().delta < 0.0f)
							countNeg++;
					}
				}

				GUI.Label (new Rect (margin, margin, Screen.width - 2 * margin, hgt), "", styles [0]);
				GUI.Label (new Rect (
					margin, 
					margin, 
					(Screen.width - 2 * margin) * ((float)countNeg / count), 
					hgt), "", styles [1]);
				GUI.Label (new Rect (margin + (Screen.width - 2 * margin) * ((float)countNeg / count), margin, 
					(Screen.width - 2 * margin) * ((float)(count - countNeg - countPos) / count), hgt), "", styles [2]);
			}
		}
    }
}
